@@ -1,13 +1,17 @@
+# -*- coding: utf-8 -*-
+
 ## Amazon S3 manager
 ## Author: Michal Ludvig <michal@logix.cz>
 ##         http://www.logix.cz/michal
 ## License: GPL Version 2
 ## Copyright: TGRMN Software and contributors
 
+from __future__ import absolute_import, division
+
 import sys
 import datetime
 import time
-import Utils
+import S3.Utils
 
 class Progress(object):
     _stdout = sys.stdout
@@ -48,7 +52,7 @@ class Progress(object):
         self.display(done_message = message)
 
     def output_labels(self):
-        self._stdout.write(u"%(source)s -> %(destination)s  %(extra)s\n" % self.labels)
+        self._stdout.write(u"%(action)s: '%(source)s' -> '%(destination)s'  %(extra)s\n" % self.labels)
         self._stdout.flush()
 
     def _display_needed(self):
@@ -70,19 +74,21 @@ class Progress(object):
             return
 
         if self.current_position == self.total_size:
-            print_size = Utils.formatSize(self.current_position, True)
+            print_size = S3.Utils.formatSize(self.current_position, True)
             if print_size[1] != "": print_size[1] += "B"
             timedelta = self.time_current - self.time_start
-            sec_elapsed = timedelta.days * 86400 + timedelta.seconds + float(timedelta.microseconds)/1000000.0
-            print_speed = Utils.formatSize((self.current_position - self.initial_position) / sec_elapsed, True, True)
+            sec_elapsed = timedelta.days * 86400 + timedelta.seconds + float(timedelta.microseconds) / 1000000.0
+            print_speed = S3.Utils.formatSize((self.current_position - self.initial_position) / sec_elapsed, True, True)
             self._stdout.write("100%%  %s%s in %.2fs (%.2f %sB/s)\n" %
                 (print_size[0], print_size[1], sec_elapsed, print_speed[0], print_speed[1]))
             self._stdout.flush()
             return
 
-        rel_position = self.current_position * 100 / self.total_size
+        rel_position = (self.current_position * 100) // self.total_size
         if rel_position >= self.last_milestone:
-            self.last_milestone = (int(rel_position) / 5) * 5
+            # Move by increments of 5.
+            # NOTE: to check: Looks like to not do what is looks like to be designed to do
+            self.last_milestone = (rel_position // 5) * 5
             self._stdout.write("%d%% ", self.last_milestone)
             self._stdout.flush()
             return
@@ -115,7 +121,7 @@ class ProgressANSI(Progress):
         timedelta = self.time_current - self.time_start
         sec_elapsed = timedelta.days * 86400 + timedelta.seconds + float(timedelta.microseconds)/1000000.0
         if (sec_elapsed > 0):
-            print_speed = Utils.formatSize((self.current_position - self.initial_position) / sec_elapsed, True, True)
+            print_speed = S3.Utils.formatSize((self.current_position - self.initial_position) / sec_elapsed, True, True)
         else:
             print_speed = (0, "")
         self._stdout.write(self.ANSI_restore_cursor_pos)
@@ -123,7 +129,7 @@ class ProgressANSI(Progress):
         self._stdout.write("%(current)s of %(total)s   %(percent)3d%% in %(elapsed)ds  %(speed).2f %(speed_coeff)sB/s" % {
             "current" : str(self.current_position).rjust(len(str(self.total_size))),
             "total" : self.total_size,
-            "percent" : self.total_size and (self.current_position * 100 / self.total_size) or 0,
+            "percent" : self.total_size and ((self.current_position * 100) // self.total_size) or 0,
             "elapsed" : sec_elapsed,
             "speed" : print_speed[0],
             "speed_coeff" : print_speed[1]
@@ -153,14 +159,14 @@ class ProgressCR(Progress):
         timedelta = self.time_current - self.time_start
         sec_elapsed = timedelta.days * 86400 + timedelta.seconds + float(timedelta.microseconds)/1000000.0
         if (sec_elapsed > 0):
-            print_speed = Utils.formatSize((self.current_position - self.initial_position) / sec_elapsed, True, True)
+            print_speed = S3.Utils.formatSize((self.current_position - self.initial_position) / sec_elapsed, True, True)
         else:
             print_speed = (0, "")
         self._stdout.write(self.CR_char)
         output = " %(current)s of %(total)s   %(percent)3d%% in %(elapsed)4ds  %(speed)7.2f %(speed_coeff)sB/s" % {
             "current" : str(self.current_position).rjust(len(str(self.total_size))),
             "total" : self.total_size,
-            "percent" : self.total_size and (self.current_position * 100 / self.total_size) or 0,
+            "percent" : self.total_size and ((self.current_position * 100) // self.total_size) or 0,
             "elapsed" : sec_elapsed,
             "speed" : print_speed[0],
             "speed_coeff" : print_speed[1]
@@ -171,4 +177,44 @@ class ProgressCR(Progress):
 
         self._stdout.flush()
 
+class StatsInfo(object):
+    """Holding info for stats totals"""
+    def __init__(self):
+        self.files = None
+        self.size = None
+        self.files_transferred = None
+        self.size_transferred = None
+        self.files_copied = None
+        self.size_copied = None
+        self.files_deleted = None
+        self.size_deleted = None
+
+    def format_output(self):
+        outstr = u""
+        if self.files is not None:
+            tmp_str = u"Number of files: %d"% self.files
+            if self.size is not None:
+                tmp_str += " (%d bytes) "% self.size
+            outstr += u"\nStats: " + tmp_str
+
+        if self.files_transferred:
+            tmp_str = u"Number of files transferred: %d"% self.files_transferred
+            if self.size_transferred is not None:
+                tmp_str += " (%d bytes) "% self.size_transferred
+            outstr += u"\nStats: " + tmp_str
+
+        if self.files_copied:
+            tmp_str = u"Number of files copied: %d"% self.files_copied
+            if self.size_copied is not None:
+                tmp_str += " (%d bytes) "% self.size_copied
+            outstr += u"\nStats: " + tmp_str
+
+        if self.files_deleted:
+            tmp_str = u"Number of files deleted: %d"% self.files_deleted
+            if self.size_deleted is not None:
+                tmp_str += " (%d bytes) "% self.size_deleted
+            outstr += u"\nStats: " + tmp_str
+
+        return outstr
+
 # vim:et:ts=4:sts=4:ai
